# -*- coding: utf-8 -*-


from imio.smartweb.common.interfaces import ILocalManagerAware
from imio.smartweb.locales import SmartwebMessageFactory as _
from collective.z3cform.datagridfield.datagridfield import DataGridFieldFactory
from collective.z3cform.datagridfield.row import DictRow
from plone import schema
from plone.app.z3cform.widget import SelectFieldWidget
from plone.autoform import directives
from plone.dexterity.content import Container
from plone.supermodel import model
from zope.interface import implementer
from zope.interface import Interface


class ILocalCategoryRowSchema(Interface):

    fr = schema.TextLine(
        title=_("French"),
        required=True,
    )

    nl = schema.TextLine(
        title=_("Dutch"),
        required=False,
    )

    de = schema.TextLine(
        title=_("Deutsch"),
        required=False,
    )

    en = schema.TextLine(
        title=_("English"),
        required=False,
    )


class IEntity(model.Schema):
    """Marker interface and Dexterity Python Schema for Entity"""

    directives.widget(zip_codes=SelectFieldWidget)
    zip_codes = schema.List(
        title=_("Zip codes and cities"),
        description=_("Choose zip codes for this entity"),
        value_type=schema.Choice(vocabulary="imio.smartweb.vocabulary.Cities"),
    )

    model.fieldset("categorization", fields=["local_categories"])
    local_categories = schema.List(
        title=_("Specific events categories"),
        description=_(
            "List of events categories values available for this entity (one per line)"
        ),
        value_type=DictRow(title="Value", schema=ILocalCategoryRowSchema),
        required=False,
    )
    directives.widget(
        "local_categories", DataGridFieldFactory, allow_reorder=True, auto_append=False
    )

    directives.read_permission(
        authorize_to_bring_event_anywhere="imio.events.core.BringEventIntoPersonnalAgenda"
    )
    directives.write_permission(
        authorize_to_bring_event_anywhere="imio.events.core.BringEventIntoPersonnalAgenda"
    )
    authorize_to_bring_event_anywhere = schema.Bool(
        title=_("Authorize to bring event anywhere"),
        description=_(
            "If selected, contributor of this entity can bring event in any agenda independently of agenda subscribing feature"
        ),
        required=False,
        default=False,
    )


@implementer(IEntity, ILocalManagerAware)
class Entity(Container):
    """Entity content type"""



# -*- coding: utf-8 -*-

from datetime import datetime
from datetime import timedelta
from eea.facetednavigation.settings.interfaces import IHidePloneLeftColumn
from imio.events.core.contents import IAgenda
from imio.events.core.contents import IEntity
from imio.smartweb.common.faceted.utils import configure_faceted
from imio.smartweb.common.vocabularies import IAmVocabulary
from imio.smartweb.common.vocabularies import TopicsVocabulary
from plone import api
from plone.event.recurrence import recurrence_sequence_ical
from plone.restapi.serializer.converters import json_compatible
from Products.CMFPlone.utils import parent
from pytz import utc
from zope.component import getMultiAdapter
from zope.i18n import translate
from zope.interface import noLongerProvides

import dateutil
import os


def get_entity_for_obj(obj):
    while not IEntity.providedBy(obj) and obj is not None:
        obj = parent(obj)
    entity = obj
    return entity


def get_agenda_for_event(event):
    obj = event
    while not IAgenda.providedBy(obj) and obj is not None:
        obj = parent(obj)
    agenda = obj
    return agenda


def get_agendas_uids_for_faceted(obj):
    if IAgenda.providedBy(obj):
        return [obj.UID()]
    elif IEntity.providedBy(obj):
        brains = api.content.find(context=obj, portal_type="imio.events.Agenda")
        return [b.UID for b in brains]
    else:
        raise NotImplementedError


def reload_faceted_config(obj, request):
    faceted_config_path = "{}/faceted/config/events.xml".format(
        os.path.dirname(__file__)
    )
    configure_faceted(obj, faceted_config_path)
    agendas_uids = "\n".join(get_agendas_uids_for_faceted(obj))
    request.form = {
        "cid": "agenda",
        "faceted.agenda.default": agendas_uids,
    }
    handler = getMultiAdapter((obj, request), name="faceted_update_criterion")
    handler.edit(**request.form)
    if IHidePloneLeftColumn.providedBy(obj):
        noLongerProvides(obj, IHidePloneLeftColumn)


def get_start_date(event):
    return datetime.fromisoformat(event["start"])


# If we want to further optimize the retrieval of a single event without using fullobjects=1
# , then this function will be useful.
def hydrate_ids_for(field_name, event, vocabulary):
    current_lang = event.get("language", "fr")
    raw_ids = event.get(field_name, [])
    result = []
    if not raw_ids:
        return raw_ids
    for term_id in raw_ids:
        term = vocabulary.getTermByToken(term_id)
        result.append(
            {
                "title": translate(term.title, target_language=current_lang),
                "token": term.token,
            }
        )
    return result


# If we want to further optimize the retrieval of a single event without using fullobjects=1
# , then this function will be useful.
# def get_gallery_images(event):
#     pass


# just expand occurences. No filtering here
def expand_occurences(events, range="min"):
    expanded_events = []
    # iam_vocabulary = IAmVocabulary()
    # topics_vocabulary = TopicsVocabulary()
    for event in events:
        if event is None:
            continue
        first_start = event.get("first_start") or event.get("start")
        first_end = event.get("first_end") or event.get("end")
        start_date = dateutil.parser.parse(first_start).astimezone(utc)
        end_date = dateutil.parser.parse(first_end).astimezone(utc)
        event["geolocation"] = {
            "latitude": event.get("latitude", ""),
            "longitude": event.get("longitude", ""),
        }
        # without fullobjects
        # event["iam"] = hydrate_ids_for("iam", event, iam_vocabulary)
        # event["topics"] = hydrate_ids_for("topics", event, topics_vocabulary)

        if event.get("image_scales", None):
            id_event = event["@id"]
            url_image = event["image_scales"]["image"][0]["download"]
            event["image_scales"]["image"][0][
                "download"
            ] = url_image  # f"{id_event}{url_image}"
            scales = event["image_scales"]["image"][0]["scales"]
            for k, v in scales.items():
                download = v["download"]
                v["download"] = f"{id_event}/{download}"
            event["image"] = event["image_scales"]["image"][0]
            del event["image_scales"]
        event["has_leadimage"] = False
        if event.get("image", None):
            event["has_leadimage"] = True
        # Ensure event start/end are in same date format than other json dates
        event["start"] = json_compatible(start_date)
        event["end"] = json_compatible(end_date)
        if event["whole_day"]:
            duration = timedelta(hours=23, minutes=59, seconds=59)
            event["end"] = json_compatible(start_date + duration)
        if not event["recurrence"]:
            expanded_events.append(event)
            continue
        # optimize query with "until" to avoid to go through all recurrences
        # if we want "future events", we get occurences to 1 years in the future
        # if we want "past events", we get occurences to 1 year in the past
        until = from_ = None
        until = start_date + timedelta(days=365)
        # for now min:max is only supported for future events
        if range == "min:max":
            from_ = datetime.now()
        elif range == "max":
            from_ = start_date - timedelta(days=365)
            until = datetime.now()
        start_dates = recurrence_sequence_ical(
            start=start_date,
            recrule=event["recurrence"],
            from_=from_,
            until=until,
        )

        for occurence_start in start_dates:
            new_event = {**event}
            start_time = datetime.combine(datetime.today(), start_date.time())
            end_time = datetime.combine(datetime.today(), end_date.time())
            duration = end_time - start_time
            new_event["start"] = json_compatible(occurence_start)
            new_event["end"] = json_compatible(occurence_start + duration)
            expanded_events.append(new_event)
    return expanded_events


def remove_zero_interval_from_recrule(recrule):
    if not recrule:
        return recrule
    recrule = recrule.replace(";INTERVAL=0", "")
    return recrule



from imio.events.core.contents import IAgenda
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.content.browser.actions import (
    DeleteConfirmationForm as BaseDeleteConfirmationForm,
)
from plone.app.content.browser.contents.delete import (
    DeleteActionView as BaseDeleteActionView,
)
from zope.i18n import translate


class DeleteConfirmationForm(BaseDeleteConfirmationForm):

    @property
    def items_to_delete(self):
        count = len(self.context.items())
        if count >= 1:
            container_to_delete = (
                _("Agenda") if IAgenda.providedBy(self.context) else _("Folder")
            )
            txt = _(
                "${val1} ${val2} can't be removed because it contains ${val3} element(s)",
                mapping={
                    "val1": container_to_delete,
                    "val2": self.context.Title(),
                    "val3": count,
                },
            )
            msg = translate(txt, context=self.request)
            api.portal.show_message(msg, self.request, type="warn")
            self.request.response.redirect(self.context.absolute_url())
            return 0
        return count


class DeleteActionView(BaseDeleteActionView):

    def action(self, obj):
        count = len(obj.items())
        if count >= 1:
            container_to_delete = (
                _("Agenda") if IAgenda.providedBy(self.context) else _("Folder")
            )
            txt = _(
                "${val1} ${val2} can't be removed because it contains ${val3} element(s)",
                mapping={
                    "val1": container_to_delete,
                    "val2": obj.Title(),
                    "val3": count,
                },
            )
            msg = translate(txt, context=self.request)
            self.errors.append(msg)
            return
        return super(DeleteActionView, self).action(obj)



# -*- coding: utf-8 -*-

from imio.smartweb.common.utils import is_log_active
from imio.events.core.utils import remove_zero_interval_from_recrule
from plone.formwidget.recurrence.browser.json_recurrence import RecurrenceView

import logging

logger = logging.getLogger("imio.events.core")


class LoggingRecurrenceView(RecurrenceView):
    @property
    def json_string(self):
        if is_log_active():
            logger.info(f"Event recurrence request: {self.request.form}")
        if "rrule" in self.request.form:
            rrule = self.request.form["rrule"]
            # INTERVAL=0 is not allowed in RFC 5545
            # See https://github.com/plone/plone.formwidget.recurrence/issues/39
            self.request.form["rrule"] = remove_zero_interval_from_recrule(rrule)
        return super(LoggingRecurrenceView, self).json_string



# -*- coding: utf-8 -*-

from imio.events.core.testing import IMIO_EVENTS_CORE_FUNCTIONAL_TESTING
from imio.events.core import vocabularies
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.testing.zope import Browser

import json
import transaction
import unittest


class TestEvent(unittest.TestCase):
    layer = IMIO_EVENTS_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.entity1 = api.content.create(
            container=self.portal,
            type="imio.events.Entity",
            id="entity1",
            title="Entity 1",
        )
        self.agenda1 = api.content.create(
            container=self.entity1,
            type="imio.events.Agenda",
            id="agenda1",
            title="Agenda 1",
        )
        self.agenda1b = api.content.create(
            container=self.entity1,
            type="imio.events.Agenda",
            id="agenda1b",
            title="Agenda 1b",
        )
        self.event1 = api.content.create(
            container=self.agenda1,
            type="imio.events.Event",
            id="event1",
            title="Event 1",
        )

        self.entity2 = api.content.create(
            container=self.portal,
            type="imio.events.Entity",
            id="entity2",
            title="Entity 2",
        )
        self.agenda2 = api.content.create(
            container=self.entity2,
            type="imio.events.Agenda",
            id="agenda2",
            title="Agenda 2",
        )
        self.event2 = api.content.create(
            container=self.agenda2,
            type="imio.events.Event",
            id="event2",
            title="Event 2",
        )

    def test_brings_event_into_agendas(self):
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        vocabularies.ENABLE_CACHE = False
        browser.open(
            f"{self.event2.absolute_url()}/@@bring_event_into_agendas_form/@@getVocabulary?name=imio.events.vocabulary.UserAgendas&field=agendas"
        )
        content = browser.contents
        results = json.loads(content).get("results")
        available_agendas_uids = [r.get("id") for r in results]
        self.assertNotIn(self.event2.selected_agendas, available_agendas_uids)

        # To be continued...
        # browser.open(f"{self.event2.absolute_url()}/@@bring_event_into_agendas_form")
        # select = browser.getControl(name="form.widgets.agendas")
        # select.value = "23eaca484039416c959a29a1c7509470"

        # button = browser.getControl(name="form.buttons.submit").click()

        # request = self.request
        # # request.form['agendas'] = "23eaca484039416c959a29a1c7509470"
        # # request.form['form.buttons.submit'] = 'Submit'
        # request.form.update({'form.widgets.agendas': '23eaca484039416c959a29a1c7509470', 'form.buttons.submit': 'Submit'})

        # request = TestRequest(
        #     form={'form.widgets.agendas': '23eaca484039416c959a29a1c7509470', 'form.buttons.submit': 'Submit'}
        # )

        # alsoProvides(request, IPloneFormLayer)
        # alsoProvides(request, IBringEventIntoAgendasForm)
        # form = BringEventIntoAgendasForm(self.event2, request)
        # form.portal_type = "imio.events.Event"
        # form.update()
        # form.widgets['agendas'].value = "23eaca484039416c959a29a1c7509470"
        # form.update()
        # data, errors = form.extractData()
        # browser.open(f"{self.event2.absolute_url()}/@@bring_event_into_agendas_form")
        # browser.getControl(name="form.buttons.submit").click()
        # response_data = json.loads(browser.contents)
        # # submit_button = form.buttons['submit']
        # # form.request.form.update({'form.widgets.agendas': '23eaca484039416c959a29a1c7509470', 'form.buttons.submit': 'Submit'})
        # # form.portal_type = "imio.events.Event"
        # # form.handle_submit(form, submit_button)
        # # form.handle_submit(submit_button, action='Submit')



# -*- coding: utf-8 -*-

from datetime import datetime
from datetime import timedelta
from datetime import timezone
from imio.events.core.utils import expand_occurences
from imio.events.core.utils import get_start_date
from imio.smartweb.common.rest.search_filters import SearchFiltersHandler
from imio.smartweb.common.utils import is_log_active
from plone import api
from plone.memoize import ram
from plone.restapi.batching import HypermediaBatch
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.search.handler import SearchHandler
from plone.restapi.search.utils import unflatten_dotted_dict
from plone.restapi.services import Service
from zope.component import getMultiAdapter

import hashlib
import json
import logging
import time

logger = logging.getLogger("imio.events.core")


class SearchFiltersGet(Service):
    """
    This is a temporary shortcut to calculate search-filters based on @events
    search results logic. We need to refactor & test (more) this module.
    """

    def reply(self):
        query = self.request.form.copy()
        if "metadata_fields" not in query:
            return {}
        query = unflatten_dotted_dict(query)
        results = EventsEndpointHandler(self.context, self.request).search(query)
        UIDs = [item["UID"] for item in results["items"]]
        query = {
            "UID": UIDs,
            "metadata_fields": ["category", "local_category", "topics"],
        }
        return SearchFiltersHandler(self.context, self.request).search(query)


class EventsEndpointGet(Service):
    def reply(self):
        query = self.request.form.copy()
        query = unflatten_dotted_dict(query)
        return EventsEndpointHandler(self.context, self.request).search(query)


class EventsEndpointHandler(SearchHandler):

    def _cache_key(func, instance, query):
        query_str = json.dumps(
            query, sort_keys=True
        )  # Tri pour éviter les variations d'ordre
        query_hash = hashlib.md5(
            query_str.encode("utf-8")
        ).hexdigest()  # Hash MD5 pour éviter une clé trop longue
        return (query_hash, time.time() // 240)

    @ram.cache(_cache_key)
    def _perform_search(self, query):
        """Cette méthode effectue la recherche et l'expansion des occurrences avant le tri."""
        if "fullobjects" in query:
            del query["fullobjects"]
        fullobjects = False
        if query.get("UID", None):
            fullobjects = True

        query["portal_type"] = "imio.events.Event"
        query["review_state"] = "published"
        query["b_size"] = 400

        if "selected_agendas" in query:
            query["selected_agendas"] = sorted(
                set(self.get_cascading_agendas(query["selected_agendas"]))
            )

        self._constrain_query_by_path(query)
        query = self._parse_query(query)
        range_type = self.request.form.get("event_dates.range")
        if range_type == "max":
            self.optimize_max_range(query)
        lazy_resultset = self.catalog.searchResults(**query)

        if "metadata_fields" not in self.request.form:
            self.request.form["metadata_fields"] = []
        self.request.form["metadata_fields"] += [
            "container_uid",
            "recurrence",
            "whole_day",
            "first_start",
            "first_end",
            "open_end",
        ]
        self.request.form["b_size"] = query["b_size"]
        self.request.form["b_start"] = 0
        results = getMultiAdapter((lazy_resultset, self.request), ISerializeToJson)(
            fullobjects=fullobjects
        )
        expanded_occurrences = expand_occurences(results.get("items"), range_type)
        return expanded_occurrences, range_type

    def search(self, query=None):
        if is_log_active():
            tps1 = time.time()
        if not query:
            return {"items": []}

        b_size = query.pop("b_size", 20)
        b_start = query.pop("b_start", 0)
        # Appel à la nouvelle méthode pour obtenir les résultats de la recherche avant le tri et l'expansion
        # expanded_occurrences, range_type, b_size, b_start = self._perform_search(query)
        expanded_occurrences, range_type = self._perform_search(query)
        sorted_occurrences = self.filter_and_sort_occurrences(
            expanded_occurrences, range_type
        )

        self.request.form["b_start"] = b_start
        self.request.form["b_size"] = b_size
        if len(expanded_occurrences) < int(b_size):
            self.request.form["b_size"] = str(len(expanded_occurrences))

        batch = HypermediaBatch(self.request, sorted_occurrences)

        if is_log_active():
            tps2 = time.time()
            logger.info(f"search =====> query : {results['@id']}")
            logger.info(f"search =====> time (total) : {tps2 - tps1}")
        results = {
            "@id": batch.canonical_url,
            "items_total": batch.items_total,
            "batching": batch.links if batch.links else None,
            "items": list(batch),
        }
        return results

    def get_cascading_agendas(self, initial_agenda):
        global_list = []

        def recursive_generator(agenda_UID):
            # obj est l'agenda
            obj = api.content.get(UID=agenda_UID)
            populating_agendas = [agenda_UID]
            global_list.append(agenda_UID)

            for rv in getattr(obj, "populating_agendas", []):
                if (
                    getattr(rv, "to_object", None)
                    and rv.to_object.UID() not in global_list
                ):
                    if api.content.get_state(rv.to_object) == "published":
                        populating_agendas.append(rv.to_object.UID())
                        global_list.append(rv.to_object.UID())
                        yield from recursive_generator(rv.to_object.UID())

            yield from populating_agendas

        return recursive_generator(initial_agenda)

    def optimize_max_range(self, query):
        now = datetime.now(timezone.utc)
        one_year_ago = now - timedelta(days=365)
        query["event_dates"] = {
            "query": [one_year_ago.isoformat(), now.isoformat()],
            "range": "min:max",
        }

    def filter_and_sort_occurrences(self, occurrences, range_type):
        if not range_type:
            return occurrences

        current_date = datetime.now(timezone.utc)
        filter_func = {
            "min": lambda occ: datetime.fromisoformat(occ["start"]) >= current_date
            or (
                datetime.fromisoformat(occ["start"])
                <= current_date
                <= datetime.fromisoformat(occ["end"])
            ),
            "max": lambda occ: datetime.fromisoformat(occ["end"]) < current_date
            or (
                datetime.fromisoformat(occ["start"])
                <= current_date
                <= datetime.fromisoformat(occ["end"])
            ),
            "min:max": lambda occ: self.is_within_range(occ),
        }.get(range_type, lambda occ: True)

        if is_log_active():
            # DEBUG : log all events occurrences that not conserved (excluded)
            excluded_occurrences = []
            filtered_occurrences = []
            for occ in occurrences:
                if filter_func(occ):
                    filtered_occurrences.append(occ)
                else:
                    excluded_occurrences.append(occ)

            if excluded_occurrences:
                logger.warning(
                    f"=====> Événements exclus ({range_type}): {excluded_occurrences}"
                )
            return sorted(
                filtered_occurrences, key=get_start_date, reverse=(range_type == "max")
            )
        else:
            return sorted(
                filter(filter_func, occurrences),
                key=get_start_date,
                reverse=(range_type == "max"),
            )

    def is_within_range(self, occurrence):
        min_date, max_date = self.request.form.get("event_dates.query", [None, None])
        if min_date and max_date:
            min_date = datetime.fromisoformat(min_date).replace(tzinfo=timezone.utc)
            max_date = datetime.fromisoformat(max_date).replace(
                hour=23, minute=59, second=59, tzinfo=timezone.utc
            )
            start_date = datetime.fromisoformat(occurrence["start"])
            end_date = datetime.fromisoformat(occurrence["end"])
            return (
                (min_date <= start_date <= max_date)
                or (start_date <= min_date and end_date >= min_date)
                or (start_date <= max_date and end_date >= max_date)
            )
        return True



# -*- coding: utf-8 -*-

from collective.taxonomy.interfaces import ITaxonomy
from AccessControl import Unauthorized
from imio.events.core.contents import IAgenda
from imio.events.core.contents import IEntity
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.memoize import ram
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from Products.CMFPlone.utils import parent
from zope.component import getSiteManager
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary
from zope.interface import provider

import time


ENABLE_CACHE = True


def _cache_key(func, context):
    user = api.user.get_current()
    return f"user_{user.getId()}_{int(time.time() / 10)}"  # Changes every 30s


class EventsCategoriesVocabularyFactory:
    def __call__(self, context=None):
        values = [
            ("stroll_discovery", _("Stroll and discovery")),
            ("flea_market_market", _("Flea market and market")),
            ("concert_festival", _("Concert and festival")),
            ("conference_debate", _("Conference and debate")),
            ("exhibition_artistic_meeting", _("Exhibition and artistic meeting")),
            ("party_folklore", _("Party and folklore")),
            ("projection_cinema", _("Projection and cinema")),
            ("trade_fair_fair", _("Trade Fair and Fair")),
            ("internships_courses", _("Internships and courses")),
            ("theater_show", _("Theater and show")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in values]
        return SimpleVocabulary(terms)


EventsCategoriesVocabulary = EventsCategoriesVocabularyFactory()


class EventsLocalCategoriesVocabularyFactory:
    def __call__(self, context=None, lang="fr"):
        if IPloneSiteRoot.providedBy(context):
            # ex: call on @types or @vocabularies from RESTAPI
            return SimpleVocabulary([])
        obj = context
        while not IEntity.providedBy(obj) and obj is not None:
            obj = parent(obj)
        if not obj.local_categories:
            return SimpleVocabulary([])

        values = {cat["fr"]: cat[lang] or cat["fr"] for cat in obj.local_categories}
        terms = [SimpleTerm(value=k, token=k, title=v) for k, v in values.items()]
        return SimpleVocabulary(terms)


EventsLocalCategoriesVocabulary = EventsLocalCategoriesVocabularyFactory()


class EventsCategoriesAndTopicsVocabularyFactory:
    def __call__(self, context=None):
        events_categories_factory = getUtility(
            IVocabularyFactory, "imio.events.vocabulary.EventsCategories"
        )

        events_local_categories_factory = getUtility(
            IVocabularyFactory, "imio.events.vocabulary.EventsLocalCategories"
        )

        topics_factory = getUtility(
            IVocabularyFactory, "imio.smartweb.vocabulary.Topics"
        )

        terms = []

        for term in events_categories_factory(context):
            terms.append(
                SimpleTerm(
                    value=term.value,
                    token=term.token,
                    title=term.title,
                )
            )

        for term in events_local_categories_factory(context):
            terms.append(
                SimpleTerm(
                    value=term.value,
                    token=term.token,
                    title=term.title,
                )
            )

        for term in topics_factory(context):
            terms.append(
                SimpleTerm(
                    value=term.value,
                    token=term.token,
                    title=term.title,
                )
            )

        return SimpleVocabulary(terms)


EventsCategoriesAndTopicsVocabulary = EventsCategoriesAndTopicsVocabularyFactory()


class AgendasUIDsVocabularyFactory:
    def __call__(self, context=None):
        portal = api.portal.get()
        brains = api.content.find(
            context=portal,
            portal_type="imio.events.Agenda",
            sort_on="breadcrumb",
        )
        terms = [
            SimpleTerm(value=b.UID, token=b.UID, title=b.breadcrumb) for b in brains
        ]
        return SimpleVocabulary(terms)


AgendasUIDsVocabulary = AgendasUIDsVocabularyFactory()


class EventTypesVocabularyFactory:
    def __call__(self, context=None):
        event_types = [
            (
                "event-driven",
                _(
                    "Event-driven (festivity, play, conference, flea market, walk, etc.)"
                ),
            ),
            (
                "activity",
                _("Activity (extracurricular, sport, workshop and course, etc.)"),
            ),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in event_types]
        return SimpleVocabulary(terms)


EventTypesVocabulary = EventTypesVocabularyFactory()


@provider(IVocabularyFactory)
class UserAgendasVocabularyFactory:

    if ENABLE_CACHE is True:

        def __call__(self, context=None):
            return self.call(context)

    else:

        @ram.cache(_cache_key)
        def __call__(self, context=None):
            return self.call(context)

    def call(self, context=None):
        site = api.portal.get()
        user = site.portal_membership.getAuthenticatedMember()
        permission = "imio.events.core: Add Event"
        # Get search query from request
        request = api.portal.getRequest()
        search_query = request.form.get("query", "").lower() if request else ""
        terms = []

        brains = api.content.find(object_provides=[IAgenda])
        for brain in brains:
            try:
                title = brain.breadcrumb.lower()
                if not search_query or search_query in title:
                    obj = brain.getObject()
                    if user.has_permission(permission, obj):
                        terms.append(
                            SimpleTerm(
                                value=brain.UID, token=brain.UID, title=brain.breadcrumb
                            )
                        )
            except Unauthorized:
                pass
        sorted_terms = sorted(terms, key=lambda x: x.title)
        return SimpleVocabulary(sorted_terms)


UserAgendasVocabulary = UserAgendasVocabularyFactory()


class EventPublicDeVocabularyFactory:
    def __call__(self, context=None):
        sm = getSiteManager()
        event_public_taxo = sm.queryUtility(
            ITaxonomy, name="collective.taxonomy.event_public"
        )
        if not event_public_taxo:
            return SimpleVocabulary([])
        categories_voca = event_public_taxo.makeVocabulary("de").inv_data
        terms = [
            SimpleTerm(value=k, token=k, title=v) for k, v in categories_voca.items()
        ]
        return SimpleVocabulary(terms)


EventPublicDeVocabulary = EventPublicDeVocabularyFactory()



# -*- coding: utf-8 -*-

from imio.events.core.viewlets.event import (
    user_is_contributor_in_entity_which_authorize_to_bring_events,
)
from imio.smartweb.common.utils import get_vocabulary
from imio.smartweb.common.widgets.select import TranslatedAjaxSelectWidget
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.autoform import directives
from plone.autoform.form import AutoExtensibleForm
from zope import schema
from z3c.form import button
from z3c.form import form
from z3c.form.button import buttonAndHandler
from plone.supermodel import model


class IBringEventIntoAgendasForm(model.Schema):
    """ """

    directives.widget(
        "agendas",
        TranslatedAjaxSelectWidget,
        vocabulary="imio.events.vocabulary.UserAgendas",
        pattern_options={"multiple": True, "minimumInputLength": 3},
    )
    directives.write_permission(agendas="cmf.SetOwnProperties")
    agendas = schema.List(
        title=_("Available agendas"),
        value_type=schema.Choice(source="imio.events.vocabulary.UserAgendas"),
        required=True,
    )


class BringEventIntoAgendasForm(AutoExtensibleForm, form.Form):
    """ """

    schema = IBringEventIntoAgendasForm
    ignoreContext = True
    enable_autofocus = False
    label = _("Add/Remove agenda(s)")

    def update(self):
        super(BringEventIntoAgendasForm, self).update()
        if user_is_contributor_in_entity_which_authorize_to_bring_events is False:
            api.portal.show_message(
                _("You don't have rights to access this page."), self.request
            )
            self.request.response.redirect(self.context.absolute_url())
            return False

    def updateWidgets(self):
        super(BringEventIntoAgendasForm, self).updateWidgets()
        selectedItems = {}
        self.selectedUID = []
        vocabulary = get_vocabulary("imio.events.vocabulary.UserAgendas")

        for term in vocabulary:
            if term.value in self.context.selected_agendas:
                self.selectedUID.append(term.value)
                selectedItems[term.value] = term.title
        self.widgets["agendas"].value = ";".join(self.selectedUID)

    @buttonAndHandler(_("Submit"))
    def handle_submit(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        if len(data.get("agendas")) < len(self.selectedUID):
            # we want to remove agenda(s) out of this event
            agendas_to_remove = list(set(self.selectedUID) - set(data.get("agendas")))
            for agenda in agendas_to_remove:
                self.context.selected_agendas.remove(agenda)
            success_message = _("Agenda(s) correctly removed.")
        else:
            # we want to add an agenda in this event
            for agenda in data.get("agendas"):
                if agenda not in self.context.selected_agendas:
                    self.context.selected_agendas.append(agenda)
            success_message = _("Agenda(s) correctly added.")

        self.context.reindexObject(idxs=["selected_agendas"])
        self.context._p_changed = 1
        self.status = success_message
        api.portal.show_message(_(self.status), self.request)

        self.request.response.redirect(self.context.absolute_url())

    @button.buttonAndHandler(_("Cancel"))
    def handleCancel(self, action):
        self.request.response.redirect(self.context.absolute_url())



# -*- coding: utf-8 -*-

from imio.smartweb.common.browser.forms import CustomAddForm
from imio.smartweb.common.browser.forms import CustomEditForm
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.dexterity.browser.add import DefaultAddView
from plone.dexterity.events import EditFinishedEvent
from plone.dexterity.i18n import MessageFactory as DMF_
from plone.z3cform import layout
from Products.statusmessages.interfaces import IStatusMessage
from zope.interface import Invalid
from z3c.form import button
from z3c.form.interfaces import WidgetActionExecutionError
from zope.event import notify


class EventCustomEditForm(CustomEditForm):
    @button.buttonAndHandler(DMF_("Save"), name="save")
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        end = data["IEventBasic.end"]
        start = data["IEventBasic.start"]
        days = (end - start).days
        # 3 years
        if days > 1095:
            self.status = self.formErrorsMessage
            msg = _("Your event must last less than 3 years.")
            raise WidgetActionExecutionError("IEventBasic.end", Invalid(msg))
        self.applyChanges(data)
        IStatusMessage(self.request).addStatusMessage(self.success_message, "info")
        self.request.response.redirect(self.nextURL())
        notify(EditFinishedEvent(self.context))


EventCustomEditView = layout.wrap_form(EventCustomEditForm)


class EventCustomAddForm(CustomAddForm):
    portal_type = "imio.events.Event"

    @button.buttonAndHandler(DMF_("Save"), name="save")
    def handleAdd(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        end = data["IEventBasic.end"]
        start = data["IEventBasic.start"]
        days = (end - start).days
        # 3 years
        if days > 1095:
            self.status = self.formErrorsMessage
            msg = _("Your event must last less than 3 years.")
            raise WidgetActionExecutionError("IEventBasic.end", Invalid(msg))
        obj = self.createAndAdd(data)
        if obj is not None:
            # mark only as finished if we get the new object
            self._finishedAdd = True
            IStatusMessage(self.request).addStatusMessage(self.success_message, "info")


class EventCustomAddView(DefaultAddView):
    form = EventCustomAddForm



# -*- coding: utf-8 -*-

from eea.facetednavigation.subtypes.interfaces import IFacetedNavigable
from imio.events.core.contents.agenda.content import IAgenda  # NOQA E501
from imio.events.core.testing import IMIO_EVENTS_CORE_INTEGRATION_TESTING  # noqa
from plone import api
from plone.api.exc import InvalidParameterError
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from z3c.relationfield import RelationValue
from z3c.relationfield.interfaces import IRelationList
from zope.component import createObject
from zope.component import getUtility
from zope.component import queryUtility
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent import Attributes
from zope.lifecycleevent import modified

import unittest


class TestAgenda(unittest.TestCase):
    layer = IMIO_EVENTS_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.authorized_types_in_agenda = [
            "imio.events.Folder",
            "imio.events.Event",
        ]
        self.unauthorized_types_in_agenda = [
            "imio.events.Agenda",
            "Document",
            "File",
            "Image",
        ]

        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.parent = self.portal
        self.entity = api.content.create(
            container=self.portal,
            type="imio.events.Entity",
            id="imio.events.Entity",
        )

    def test_ct_agenda_schema(self):
        fti = queryUtility(IDexterityFTI, name="imio.events.Agenda")
        schema = fti.lookupSchema()
        self.assertEqual(IAgenda, schema)

    def test_ct_agenda_fti(self):
        fti = queryUtility(IDexterityFTI, name="imio.events.Agenda")
        self.assertTrue(fti)

    def test_ct_agenda_factory(self):
        fti = queryUtility(IDexterityFTI, name="imio.events.Agenda")
        factory = fti.factory
        obj = createObject(factory)

        self.assertTrue(
            IAgenda.providedBy(obj),
            "IAgenda not provided by {0}!".format(
                obj,
            ),
        )

    def test_ct_agenda_adding(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        obj = api.content.create(
            container=self.entity,
            type="imio.events.Agenda",
            id="imio.events.Agenda",
        )

        self.assertTrue(
            IAgenda.providedBy(obj),
            "IAgenda not provided by {0}!".format(
                obj.id,
            ),
        )

        self.assertTrue(IFacetedNavigable.providedBy(obj))

        parent = obj.__parent__
        self.assertIn("imio.events.Agenda", parent.objectIds())

        # check that deleting the object works too
        api.content.delete(obj=obj)
        self.assertNotIn("imio.events.Agenda", parent.objectIds())

    def test_ct_agenda_globally_addable(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = queryUtility(IDexterityFTI, name="imio.events.Agenda")
        self.assertFalse(
            fti.global_allow, "{0} is not globally addable!".format(fti.id)
        )

    def test_ct_agenda_filter_content_type_true(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = queryUtility(IDexterityFTI, name="imio.events.Agenda")
        portal_types = self.portal.portal_types
        parent_id = portal_types.constructContent(
            fti.id,
            self.entity,
            "imio.events.Agenda_id",
            title="imio.events.Agenda container",
        )
        folder = self.entity[parent_id]
        for t in self.unauthorized_types_in_agenda:
            with self.assertRaises(InvalidParameterError):
                api.content.create(
                    container=folder,
                    type=t,
                    title="My {}".format(t),
                )
        for t in self.authorized_types_in_agenda:
            api.content.create(
                container=folder,
                type=t,
                title="My {}".format(t),
            )
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        with self.assertRaises(InvalidParameterError):
            api.content.create(
                container=folder,
                type="imio.events.Entity",
                title="My Entity",
            )

    def test_populating_agendas(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        agenda = api.content.create(
            container=self.entity,
            type="imio.events.Agenda",
            id="agenda",
        )
        event = api.content.create(
            container=agenda,
            type="imio.events.Event",
            id="event",
        )
        entity2 = api.content.create(
            container=self.portal,
            type="imio.events.Entity",
            id="entity2",
        )
        agenda2 = api.content.create(
            container=entity2,
            type="imio.events.Agenda",
            id="agenda2",
        )
        event2 = api.content.create(
            container=agenda2,
            type="imio.events.Event",
            id="event2",
        )
        entity3 = api.content.create(
            container=self.portal,
            type="imio.events.Entity",
            id="entity3",
        )
        agenda3 = api.content.create(
            container=entity3,
            type="imio.events.Agenda",
            id="agenda3",
        )
        folder = api.content.create(
            container=agenda3,
            type="imio.events.Folder",
            id="folder",
        )
        event3 = api.content.create(
            container=folder,
            type="imio.events.Event",
            id="event3",
        )

        # Add new agenda + subscription to existing agenda.
        intids = getUtility(IIntIds)

        agenda4 = api.content.create(
            container=self.entity,
            type="imio.events.Agenda",
            id="imio.events.Agenda4",
            populating_agendas=[RelationValue(intids.getId(agenda))],
        )
        self.assertIn(agenda4.UID(), event.selected_agendas)
        api.content.delete(agenda4)

        # Link agenda2 (all these events) to our object "agenda".
        api.relation.create(
            source=agenda, target=agenda2, relationship="populating_agendas"
        )
        modified(agenda, Attributes(IRelationList, "populating_agendas"))
        # So agenda.uid() can be find on event2
        self.assertIn(agenda.UID(), event2.selected_agendas)

        moving_event = api.content.create(
            container=agenda2,
            type="imio.events.Event",
            id="moving_event",
        )
        self.assertIn(agenda.UID(), moving_event.selected_agendas)
        # We move an event from one agenda to another
        api.content.move(moving_event, agenda3)
        self.assertNotIn(agenda.UID(), moving_event.selected_agendas)

        # Clear linking agendas out of our object "agenda".
        api.relation.delete(source=agenda, relationship="populating_agendas")
        modified(agenda, Attributes(IRelationList, "populating_agendas"))
        # So agenda.uid() can not be find on event2
        self.assertNotIn(agenda.UID(), event2.selected_agendas)

        # First, link agenda2 and agenda3
        api.relation.create(
            source=agenda, target=agenda2, relationship="populating_agendas"
        )
        api.relation.create(
            source=agenda, target=agenda3, relationship="populating_agendas"
        )
        modified(agenda, Attributes(IRelationList, "populating_agendas"))
        # Assert link is OK
        self.assertIn(agenda.UID(), event2.selected_agendas)
        self.assertIn(agenda.UID(), event3.selected_agendas)

        # Next, we delete agenda so we remove this agenda.UID() out of events.
        api.content.delete(agenda)
        self.assertNotIn(agenda.UID(), event2.selected_agendas)
        self.assertNotIn(agenda.UID(), event3.selected_agendas)
